#include "MPU6050.c"        // MPU6050 Ktphanesi

// Baglantilar
sbit LED    at LATA4_bit;  // Dahili LED RA4 Pinine Bagli

// Global Degiskeler
int accelX, accelY, accelZ, temp, gyroX, gyroY, gyroZ;

// HC-06 bluetooth modlne Veri gnderme fonksiyonu
void SendHC06(int value){
  char txt[7];              // IntToStr fonksiyonu iin txt dizisi
  IntToStr(value,txt);      // value degerini stringe cevir ve txt dizisine yaz
  UART_Write_Text(txt);     // txt dizisini uart zerinden gnder
  UART_Write_Text(" ");     // Bosluk karakteri gnder
}

void main() {
  // Port Giris-ikis Seimi (0:ikis, 1:Giris)
  TRISA=0b00000000; // [][][][LED][][][][]
  TRISB=0b00000000; // [][][][][][][SCK][SDA]
  TRISC=0b10000000; // [RX][TX][][][][][][]
  TRISD=0b00000000; // [][][][][][][][]
  TRISE=0b00000000; // [][][][][][][][]

  // Port Analog-Dijital Giris Seimi (0:Dijital, 1:Analog)
  ANSELA=0b00000000; // [][][][][][][][]
  ANSELB=0b00000000; // [][][][][][][][]
  ANSELC=0b00000000; // [][][][][][][][]
  ANSELD=0b00000000; // [][][][][][][][]
  ANSELE=0b00000000; // [][][][][][][][]

  // Port Temizleme
  LATA=0x00;
  LATB=0x00;
  LATC=0x00;
  LATD=0x00;
  LATE=0x00;

  UART1_Init(9600);
  
  // I2C Hazirlama
  I2C1_Init(400000); // MPU6050 ktphanesi dahili I2C donanimi ile alismaktadir.
  MPU6050_Init();    // MPU6050 hazirlama
  
  // Kalibrasyon
  MPU6050_Read(&accelX, &accelY, &accelZ, &temp, &gyroX, &gyroY, &gyroZ);       // Sensr verilerini oku
  MPU6050_SetOffset(accelX,accelY,accelZ,gyroX,gyroY,gyroZ);                    // Okunan degerleri offset degerleri ile gncelle
  
  // Sonsuz Dongu
  while(1){
    delay_ms(100);                                                              // 100ms bekleme
    
    MPU6050_Read(&accelX, &accelY, &accelZ, &temp, &gyroX, &gyroY, &gyroZ);     // Sensr verilerini oku

    SendHC06(accelX);                                                           // X ekseni accel verisini gnder
    SendHC06(accelY);                                                           // Y ekseni accel verisini gnder
    SendHC06(temp);                                                             // Sicaklik verisini gnder
    SendHC06(gyroX);                                                            // X ekseni gyro verisini gnder
    SendHC06(gyroY);                                                            // Y ekseni gyro verisini gnder
    SendHC06(gyroZ);                                                            // Z ekseni gyro verisini gnder
    UART_Write_Text("\r\n");                                                    // CRLF karakterlerini gndererek veri gnderimini tamamla
    
    LED=!LED;                                                                   // LED durumunu tersle
  }
}